--[[
Author: Starinnia
CPR is a combo points display addon based on Funkydude's BasicComboPoints
Poison.lua - A module for tracking deadly poison stacks
$Date: 2010-07-20 20:03:56 +0000 (Tue, 20 Jul 2010) $
$Revision: 95 $
Project Version: r96
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

local cpr = LibStub("AceAddon-3.0"):GetAddon("ComboPointsRedux")
local modName = "Deadly Poison"
local Poison = cpr:NewModule(modName, "AceConsole-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
Poison.abbrev = "DP"
local strfind = string.find
local UnitDebuff = UnitDebuff

function Poison:OnInitialize()
	self.graphics = cpr:MakeGraphicsFrame(modName)
	self.text = cpr:MakeTextFrame(modName)
end

function Poison:OnEnable()
	if not cpr.db.profile.modules[modName].disableGraphics then
		self.graphics:Show()
	end
	if not cpr.db.profile.modules[modName].disableText then
		self.text:Show()
	end
	
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function Poison:OnDisable()
	self.graphics:Hide()
	self.text:Hide()
	self:UnregisterEvent("UNIT_AURA")
	self:UnregisterEvent("PLAYER_TARGET_CHANGED")
end

local function getPoisonStack()
	local icon, count, owner
	local i = 1
	
	if UnitExists("target") then
		while UnitDebuff("target", i) ~= nil do
			_, _, texture, count, _, _, _, owner = UnitDebuff("target", i)
			if owner == "player" and strfind(texture, "Ability_Rogue_DualWeild") then
				return count
			end
			i = i + 1
		end
	end
	
	return 0
end

function Poison:UNIT_AURA(event, unit)
	if unit ~= "target" then return end
	
	if UnitIsDead("target") then
		self:UpdateDeadlyPoison(0)
		return
	end
		
	self:UpdateDeadlyPoison(getPoisonStack())
end

function Poison:PLAYER_TARGET_CHANGED()
	if UnitIsDead("target") then
		self:UpdateDeadlyPoison(0)
		return
	end
	
	self:UpdateDeadlyPoison(getPoisonStack())
end

function Poison:UpdateDeadlyPoison(stacks)
	local r, g, b = cpr:GetColorByPoints(modName, stacks)
	
	if stacks == 0 then
		for i = 1, 5 do
			self.graphics.points[i]:Hide()
		end
		stacks = ""
	elseif stacks == 5 then
		for i = stacks, 1, -1 do
			self.graphics.points[i].icon:SetVertexColor(r, g, b)
			self.graphics.points[i]:Show()
		end
	else
		for i = stacks, 1, -1 do
			self.graphics.points[i].icon:SetVertexColor(r, g, b)
			self.graphics.points[i]:Show()
		end
		for j = 5, stacks+1, -1 do
			self.graphics.points[j]:Hide()
		end
	end
	
	self.text:SetNumPoints(stacks)
end

--Set handler for the module options
function Poison:Set(info, ...)
	if info.type == "color" then
		local color = cpr.db.profile.modules[modName][info.arg]
		color[1], color[2], color[3] = ...
	else
		cpr.db.profile.modules[modName][info.arg] = ...
	end
	self:UpdateSettings()
end

--Get handler for the module options
function Poison:Get(info, ...)
	if info.type == "color" then
		return unpack(cpr.db.profile.modules[modName][info.arg])
	else
		return cpr.db.profile.modules[modName][info.arg]
	end
end

local basepath = [[Interface\Addons\ComboPointsRedux\Icons\]]
--Update is invoked by the Set handler after a change
function Poison:UpdateSettings(isCallback)
		local db = cpr.db.profile.modules[modName]
	
	--graphics
	if not db.disableGraphics then
		--change icon textures
		--update icon Alpha
		for i = 1, 5 do
			self.graphics.points[i].icon:SetTexture(basepath..db.icon)
			self.graphics.points[i]:SetAlpha(db.graphicsAlpha)
			self.graphics.points[i]:SetWidth(25*db.scale)
			self.graphics.points[i]:SetHeight(25*db.scale)
			self.graphics.points[i]:ClearAllPoints()
		end
		
		--adjust container scale
		if db.orientation == "v" then
			self.graphics:SetHeight(145*db.scale)
			self.graphics:SetWidth(25*db.scale)
		else
			self.graphics:SetWidth(145*db.scale)
			self.graphics:SetHeight(25*db.scale)
		end
		
		--adjust for orientation changes (this updates spacing too)
		local offset = db.spacing*db.scale
		if db.orientation == "v" then
			self.graphics.points[1]:SetPoint("BOTTOM", self.graphics, "BOTTOM", 0, 0)
			self.graphics.points[2]:SetPoint("BOTTOM", self.graphics.points[1], "TOP", 0, offset)
			self.graphics.points[3]:SetPoint("BOTTOM", self.graphics.points[2], "TOP", 0, offset)
			self.graphics.points[4]:SetPoint("BOTTOM", self.graphics.points[3], "TOP", 0, offset)
			self.graphics.points[5]:SetPoint("BOTTOM", self.graphics.points[4], "TOP", 0, offset)
		else
			self.graphics.points[1]:SetPoint("LEFT", self.graphics, "LEFT", 0, 0)
			self.graphics.points[2]:SetPoint("LEFT", self.graphics.points[1], "RIGHT", offset, 0)
			self.graphics.points[3]:SetPoint("LEFT", self.graphics.points[2], "RIGHT", offset, 0)
			self.graphics.points[4]:SetPoint("LEFT", self.graphics.points[3], "RIGHT", offset, 0)
			self.graphics.points[5]:SetPoint("LEFT", self.graphics.points[4], "RIGHT", offset, 0)
		end
		
		--update strata
		self.graphics:SetFrameStrata(db.strata)
		
		--update locked status
		if not cpr.db.profile.locked then
			self.graphics:SetBackdropColor(1,1,1,1)
			self.graphics:EnableMouse(true)
			self.graphics:SetMovable(true)
			self.graphics.points[1]:Show()
			self.graphics.points[2]:Show()
		else
			self.graphics:EnableMouse(false)
			self.graphics:SetMovable(false)
		end
	end
	
	--text
	if not db.disableText then
		--update font, font size, alpha and strata
		self.text.count:SetShadowColor(0, 0, 0, db.textAlpha)
		self.text.count:SetFont(LSM:Fetch("font", db.font), db.fontsize)
		self.text.count:SetTextColor(unpack(db.textColor))
		self.text.count:SetAlpha(db.textAlpha)
		self.text:SetFrameStrata(db.textStrata)
		
		--update locked status
		if not cpr.db.profile.locked then
			self.text:SetBackdropColor(1,1,1,1)
			self.text:EnableMouse(true)
			self.text:SetMovable(true)
			self.text.count:SetText(self.abbrev)
		else
			self.text:EnableMouse(false)
			self.text:SetMovable(false)
		end
	end
	
	--if this call is from a profile callback then we also need to reset the anchors
	if isCallback then
		if db.textPos.x then
			self.text:SetPoint(db.textPos.point, UIParent, db.textPos.anchor, db.textPos.x, db.textPos.y)
		else
			self.text:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
		
		if db.graphicsPos.x then
			self.graphics:SetPoint(db.graphicsPos.point, UIParent, db.graphicsPos.anchor, db.graphicsPos.x, db.graphicsPos.y)
		else
			self.graphics:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
	end
end

function Poison:ResetSettings()
	local db = cpr.db.profile.modules[modName]
	self.text:SetPoint("CENTER", UIParent, "CENTER")
	self.graphics:SetPoint("CENTER", UIParent, "CENTER")
	
	self.graphics:SetWidth(145*db.scale)
	self.graphics:SetHeight(25*db.scale)
	
	for i = 1, 5 do
		self.graphics.points[i].icon:SetTexture(basepath..db.icon)
		self.graphics.points[i]:SetWidth(25*db.scale)
		self.graphics.points[i]:SetHeight(25*db.scale)
		self.graphics.points[i]:ClearAllPoints()
	end
	
	local offset = db.spacing*db.scale
	self.graphics.points[1]:SetPoint("LEFT", self.graphics, "LEFT", 0, 0)
	self.graphics.points[2]:SetPoint("LEFT", self.graphics.points[1], "RIGHT", offset, 0)
	self.graphics.points[3]:SetPoint("LEFT", self.graphics.points[2], "RIGHT", offset, 0)
	self.graphics.points[4]:SetPoint("LEFT", self.graphics.points[3], "RIGHT", offset, 0)
	self.graphics.points[5]:SetPoint("LEFT", self.graphics.points[4], "RIGHT", offset, 0)
	
	self.graphics:SetBackdropColor(1,1,1,1)
	self.graphics:EnableMouse(true)
	self.graphics:SetMovable(true)
	self.graphics.points[1]:Show()
	self.graphics.points[2]:Show()
	
	self.text:SetBackdropColor(1,1,1,1)
	self.text:EnableMouse(true)
	self.text:SetMovable(true)
	self.text.count:SetText(self.abbrev)
end
